################################################################################
#  Replace ... by the location where scripts and datasets are saved            #
################################################################################

filepath=".../"

################################################################################
#  Source/load all scripts, librairies and datasets                            #
################################################################################

library(survival)
library(crq)

source(paste(filepath,"Chapter7_functions.r",sep=""))

load(paste(filepath,"Chapter7.rdata",sep=""))


################################################################################
# Myeloma data                                                                 # 
#                                                                              #
#                                                                              #
# source: Krall, Uthoff, and Harley (1975)                                     #
#                                                                              #
#                                                                              #
# list of variables                                                            #
# -----------------                                                            #
                                                                               #
# PID = patient ID (observation no)                                            #
# time = survival time (since diagnosis)                                       #
# status = censoring indicator (0=censored, 1=observed)                        #
# haemoglobin = haemoglobin                                                    #
# platelets = platelets at diagnosis (0=none, 1=present)                       #
# infection = infection (0=none, 1=present)                                    #
# age = age in years                                                           #
# sex = sex                                                                    #
# log wbc = log white blood cells at diasgnosis                                #
# frac = fractures at dignaosis (0=none, 1=present)                            #
# logbm = log percantage of plasmal cells in bone marrow                       #
# pct.lymph = percetage of lymphocytes at diagnosis                            #
# pctmyel.cell = percentage myeloid cells at diagnosis                         #
# proteinuria =  proteinuria at diagnosis                                      #
# protein.urine  = protein in urine                                            #
# total.serum.prot total serum protein                                         #
# serum.globin = serum globin gm%                                              #
# serum.calcium = serum calcium at diagnosis                                   #
# logbun = log BUN (to base 10)                                                #
################################################################################


 
# reduced myeloma data  
 
myeloma$logbun=10^myeloma$logbun
myeloma.reduced=myeloma[,c(2,3,4,18,19)]
names(myeloma.reduced)[5]="bun" 

# reduced myeloma data - observation 40 removed 

myeloma.reduced_40=myeloma.reduced[-40,]
 

########################
# Table 7.2            #
########################

# model-based variance - all data

fit.PLE=coxph( Surv(time, status) ~ bun + hgb + serum.calcium,data=myeloma.reduced)  
summary(fit.PLE)


# model-based variance - observation 40 removed

fit.PLE=coxph(Surv(time, status) ~ bun + hgb + serum.calcium,data=myeloma.reduced_40)  
summary(fit.PLE)

########################
# Table 7.3            #
########################


 


# LW formula - all data

fit.PLE.LW=coxph( Surv(time, status) ~ bun + hgb + serum.calcium,robust=TRUE,data=myeloma.reduced)    
summary(fit.PLE.LW)

# NB: the SE's are slightly different to the ones reported in the book;
# this is because SAS was used for the classical analysis and the
# implementation of the LW formula in R is slighltly different. 
# This minor discrepancy does not change the interpretation.

# LW formula - observation 40 removed

 


fit.PLE.LW=coxph(Surv(time, status) ~ bun + hgb + serum.calcium,robust=TRUE,data=myeloma.reduced_40)   
summary(fit.PLE.LW)


########################
# Table 7.4            #
########################
 
 
 
x=myeloma.reduced_40[,-c(1,2)]
fit.ARE=rcoxph(myeloma.reduced_40$time,myeloma.reduced_40$status,x,wt.type="exponential",quant=.80)
fit.ARE$coefficients

########################
# Figure 7.1           #
########################

K=quantile(fit.ARE$Lambda.rob*exp(fit.ARE$xbeta.rob),probs=c(.80))
nb=seq(1,65,1)
logwt0=-log(fit.ARE$wt[myeloma.reduced$status==0])*K
logwt1=-log(fit.ARE$wt[myeloma.reduced$status==1])*K
n0=nb[myeloma.reduced$status==0]
n1=nb[myeloma.reduced$status==1]
plot(-log(fit.ARE$wt)*K,ylab="-K log(weight)",xlab="Case number",xlim=c(0,65),ylim=c(0,8),type="n")
points(n0,logwt0,pch=21)
points(n1,logwt1,pch=19)
abline(h=K)
legend(3,7.5,c("Censored","Dead"),pch=c(21,19))
text(-3.75,1.4,"K=1.12",pos=4)
text(49,7.8,"case 48",pos=4)
text(41,6.1,"case 40",pos=4)


################################################################################
# Veteran data                                                                 #
#                                                                              #
#                                                                              #
# Source: Kalfleisch and Prentice (1980), The statistical Analysis of Failure  #
# time data; pp 223-224                                                        #
#                                                                              #
#                                                                              #
# list of variables                                                            #
# -----------------                                                            #
#                                                                              #
# PID       = patient ID (observation no)                                      #
# karnofsky = karnosfsky p[erformance status                                   #
# dduration = disease duration                                                 #
# age       = age in years                                                     #
# ptherapy  = prior therapy (0/10)                                             #
# squamous  = squamous cell type (0/1)                                         #
# small     = squamous cell type (0/1)                                         #
# adeno     = adeno cell type (0/1) - reference for cell type =large           #
# treatment = treament (0=standard, 1= experimental                            #
# time      = survival time in days                                            #
# status    = censoring indicator (0=censored, 1=observed)                     #
################################################################################


########################
# Table 7.5            #
########################

# classical fit (PLE)
# -------------------


# model-based variance
veterans[,5]=veterans[,5]/10 # prior therapy coded on 0/1

 

fit.PLE=coxph(Surv(time, status) ~ karnofsky + dduration + age +
ptherapy + squamous + small + adeno + treatment,data=veterans)
summary(fit.PLE)


# robust fit (ARE) with exponential weights
# -----------------------------------------
 
x=veterans[,2:9]
fit.ARE=rcoxph(veterans$time,veterans$status,x,wt.type="exponential",quant=.90)
fit.ARE$coeff

########################
# Figure 7.3           #
########################

# plot of the weight vs case number
# --------------------------------

K=quantile(fit.ARE$Lambda.rob*exp(fit.ARE$xbeta.rob),probs=c(.90))
nb=seq(1,137,1)
logwt0=-log(fit.ARE$wt[veterans$status==0])*K
logwt1=-log(fit.ARE$wt[veterans$status==1])*K
n0=nb[veterans$status==0]
n1=nb[veterans$status==1]
plot(-log(fit.ARE$wt)*K,ylab="-Klog(weight)",xlab="Case number",xlim=c(0,142),ylim=c(0,14),type="n")
points(n0,logwt0,pch=21)
points(n1,logwt1,pch=19)
abline(h=K)
legend(105,13.5,c("Censored","Dead"),pch=c(21,19))

text(125,2.6,"K=2.30",pos=4)
text(20,7.2,"case 17",pos=4)
text(47,14.2,"case 44",pos=4)

########################
# Figure 7.4           #
########################


# plot of the transformed matrix weights versus survival time (time needs to be sorted)
# ------------------------------------------------------------------------------------


fit.ARE=rcoxph(veterans$time,veterans$status,x,wt.type="exponential",quant=.90,long.output=TRUE)

plot(sort(veterans$time),-K*log(fit.ARE$wt.mat[131,]),type="l",lty=1,xlab="Survival time" ,ylab="-K log(matrix weight)")

# line stops at time 392 (case 44 no longer in the risk set beyond that point)

lines(sort(veterans$time),-K*log(fit.ARE$wt.mat[132,]),lty=2)
lines(sort(veterans$time),-K*log(fit.ARE$wt.mat[133,]),lty=3)
lines(sort(veterans$time),-K*log(fit.ARE$wt.mat[134,]),lty=4)
lines(sort(veterans$time),-K*log(fit.ARE$wt.mat[135,]),lty=5)
lines(sort(veterans$time),-K*log(fit.ARE$wt.mat[136,]),lty=6)
lines(sort(veterans$time),-K*log(fit.ARE$wt.mat[137,]),lty=7)

# observation ranked last stays until the end (time=999)

abline(h=2,lty=10)
abline(h=4,lty=10)
abline(h=6,lty=10)
text(875,2.4,"p=.14")
text(875,4.5,"p=.02")
text(875,6.5,"p=.0025")
text(520,14,"Case 44 line")


########################
# Figure 7.6           #
########################

# code for new CRQ downloaded from R - working on R 2.8.1
# ------------------------------------------------------

# increase memory to compute the confidence band when R (or n) is large
# - unexpected crashesmay occur if you don't

# follow the steps below to increase memory:
# - if you have a shortcut on your desktop for R (or somewhere else),
# then right click on the icon and bring up the Properties window
# you will see an entry for target with the path to the executable filled in. 
# - After that path (and outside of the quotes) add the command-line argument 
# as suggested in RWin FAQ 2 On my PC, for example it would look like this: 
# "C:\Program Files\R\R-2.8.1\bin\Rgui.exe" --max-mem-size=1024M

vet.crq <- crq(formula = Surv(log(time), status) ~ karnofsky + dduration + age + 
ptherapy + squamous + small + adeno + treatment, method="grid",data=veterans)

vet.cox <- coxph(formula = Surv(time, status) ~ karnofsky + dduration + age + 
ptherapy + squamous + small + adeno + treatment,data=veterans)


sfit=summary.crq(vet.crq,taus=1:19/20,R=600) # R=the number of replicates
plot(sfit,Coxvet.cox=vet.cox)

# NB: the plots and confidence bands are slightly different to what is presented in the book
# (see Section 7.6.5). A beta version of CRQ with a slightly different implentation
# was used to produce these plots. It is recommended to use CRQ or quantreg downloaded 
# from the R website where they will be regularly updated.


########################
# Table 7.6            #
########################

summary(vet.crq,taus=c(0.25,0.50,0.75),R=1000)
